from importlib import import_module


def import_string(dotted_path):
    """
    Import a dotted module path and return the attribute/class designated by the
    last name in the path. Raise ImportError if the import failed.
    """
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)
    except ValueError as err:
        raise ImportError("%s doesn't look like a module path" %
                          dotted_path) from err

    module = import_module(module_path)

    try:
        return getattr(module, class_name)
    except AttributeError as err:
        raise ImportError('Module "%s" does not define a "%s" attribute/class' % (
            module_path, class_name)
        ) from err


def import_string_list(dotted_path_list=[]):
    """
       dotted_path_list can be a list of dotted strings or
       mixed dotted strings and modules
    """
    module_list = []
    for handler in dotted_path_list:
        module = handler
        if not callable(handler):
            module = import_string(handler)

        module_list.append(module)
    return module_list



import os
import temlogger


def clean_temlogger_config():
    environments_to_clean = [
        'TEMLOGGER_PROVIDER',
        'TEMLOGGER_URL',
        'TEMLOGGER_PORT'
        'TEMLOGGER_ENVIRONMENT'
    ]
    for env in environments_to_clean:
        if env in os.environ:
            del os.environ[env]

    temlogger.config.clear()


def add_tracker_id_to_message(message):
    message['tracker_id_global'] = 'tracker_id_value_global'
    return message



import logging
import os
import unittest
import temlogger

from unittest import mock

from .base import clean_temlogger_config


class TestDefaultLogger(unittest.TestCase):

    def setUp(self):
        """
        Clean config between tests
        """
        clean_temlogger_config()

    def test_get_default_logger_when_logging_provider_is_not_set(self):
        logger = temlogger.getLogger('test')

        self.assertTrue(isinstance(logger, logging.Logger))
        self.assertEqual(logger.logging_provider, 'default')
        self.assertEqual(len(logger.handlers), 0)


class TestLogstashLogger(unittest.TestCase):

    def setUp(self):
        """
        Clean config between tests
        """
        clean_temlogger_config()

    def test_switch_logger(self):
        logger = temlogger.getLogger('switch-logger-1')
        self.assertTrue(isinstance(logger, logging.Logger))
        self.assertEqual(logger.logging_provider, 'default')

        temlogger.config.set_provider('logstash')
        logger = temlogger.getLogger('switch-logger-1')
        self.assertTrue(isinstance(logger, logging.Logger))
        self.assertEqual(logger.logging_provider, 'logstash')

        temlogger.config.set_provider('default')
        logger = temlogger.getLogger('switch-logger-1')
        self.assertTrue(isinstance(logger, logging.Logger))
        self.assertEqual(logger.logging_provider, 'default')

    def test_get_logstash_logger_passing_envs_by_environ(self):
        os.environ['TEMLOGGER_PROVIDER'] = 'logstash'
        os.environ['TEMLOGGER_URL'] = 'localhost'
        os.environ['TEMLOGGER_PORT'] = '5000'

        logger = temlogger.getLogger('logstash-2')
        self.assertEqual(logger.logging_provider, 'logstash')

    def test_get_logstash_logger_passing_envs_as_parameter(self):
        temlogger.config.set_provider('logstash')
        temlogger.config.set_url('localhost')
        temlogger.config.set_port('5000')
        temlogger.config.set_environment('staging')

        logger = temlogger.getLogger('logstash-3')
        self.assertEqual(logger.logging_provider, 'logstash')

    def test_get_logstash_logger_and_log_info(self):
        """"""
        temlogger.config.set_provider('logstash')
        temlogger.config.set_url('localhost')
        temlogger.config.set_port('5000')

        logger = temlogger.getLogger('logstash-3')

        self.assertEqual(len(logger.handlers), 1)
        self.assertEqual(logger.logging_provider, 'logstash')

        logger._log = mock.Mock()

        logger.info('Logstash log')

        logger._log.assert_called_once_with(logging.INFO, 'Logstash log', ())


class TestStackDriverLogger(unittest.TestCase):
    """
    Based on official tests:
    https://github.com/googleapis/google-cloud-python/blob/master/logging/tests/unit/test_logger.py
    """

    def setUp(self):
        """
        Clean config between tests
        """
        clean_temlogger_config()

    @mock.patch("google.cloud.logging.Client")
    def test_get_stackdriver_logger_passing_envs_by_environ(self, mocked_cls):
        os.environ['TEMLOGGER_PROVIDER'] = 'stackdriver'

        logger = temlogger.getLogger('stackdriver-1')

        self.assertTrue(isinstance(logger, logging.Logger))
        self.assertEqual(logger.logging_provider, 'stackdriver')

    @mock.patch("google.cloud.logging.Client")
    def test_get_stackdriver_logger_passing_envs_as_parameter(self, mocked_cls):
        """"""
        temlogger.config.set_provider('stackdriver')

        logger = temlogger.getLogger('stackdriver-2')
        self.assertEqual(logger.logging_provider, 'stackdriver')

    @mock.patch("google.cloud.logging.Client")
    def test_get_stackdriver_logger_and_log_info(self, mocked_cls):
        """"""
        temlogger.config.set_provider('stackdriver')

        logger = temlogger.getLogger('stackdriver-2')

        self.assertEqual(len(logger.handlers), 1)
        self.assertEqual(logger.logging_provider, 'stackdriver')

        logger._log = mock.Mock()

        logger.info('StackDriver log')

        logger._log.assert_called_once_with(
            logging.INFO, 'StackDriver log', ())



import itertools

from logstash.formatter import LogstashFormatterBase
from temlogger import config

from ..helpers import import_string_list


class FormatterBase(LogstashFormatterBase):

    def __init__(self, fqdn=False, environment='', event_handlers=[],
                 *args, **kwargs):
        super().__init__(message_type='', fqdn=fqdn, *args, **kwargs)
        self.environment = environment
        self.event_handlers = import_string_list(event_handlers)

    def format_with_handlers(self, message):
        default_handlers = config.get_event_handlers()
        handlers = itertools.chain(default_handlers, self.event_handlers)

        for handle in handlers:
            message = handle(message)

        return message

    def format(self, record):
        # Create message dict
        message = {
            '@timestamp': self.format_timestamp(record.created),
            'message': record.getMessage(),
            'host': self.host,
            'path': record.pathname,
            'environment': self.environment,

            # Extra Fields
            'level': record.levelname,
            'logger_name': record.name,
        }

        # Add extra fields
        message.update(self.get_extra_fields(record))

        # If exception, add debug info
        if record.exc_info:
            message.update(self.get_debug_fields(record))

        message = self.format_with_handlers(message)

        return message



import logging
import os


from .helpers import import_string_list


class LoggingProvider:
    STACK_DRIVER = 'stackdriver'
    LOGSTASH = 'logstash'
    DEFAULT = 'default'


class LoggingConfig:
    _provider = ''
    _url = ''
    _port = ''
    _environment = ''
    _event_handlers = []

    def set_provider(self, value):
        self._provider = value

    def get_provider(self):
        return self._provider.lower() or os.getenv('TEMLOGGER_PROVIDER', '').lower()

    def set_url(self, value):
        self._url = value

    def get_url(self):
        return self._url or os.getenv('TEMLOGGER_URL', '')

    def set_port(self, value):
        self._port = value

    def get_port(self):
        return self._port or os.getenv('TEMLOGGER_PORT', '')

    def set_environment(self, value):
        self._environment = value

    def get_environment(self):
        return self._environment or os.getenv('TEMLOGGER_ENVIRONMENT', '')

    def get_event_handlers(self):
        return self._event_handlers

    def setup_event_handlers(self, event_handlers=[]):
        self._event_handlers = import_string_list(event_handlers)

    def clear(self):
        self._provider = ''
        self._url = ''
        self._port = ''
        self._environment = ''
        self._event_handlers = []


class LoggerManager:

    def __init__(self):
        self.logger_map = {
            LoggingProvider.LOGSTASH: self.get_logger_logstash,
            LoggingProvider.STACK_DRIVER: self.get_logger_stackdriver,
            LoggingProvider.DEFAULT: self.get_logger_default,
        }

    def get_logger(self, name, event_handlers=[]):
        logging_provider = config.get_provider()

        logger = logging.getLogger(name)
        has_log_provider = hasattr(logger, 'logging_provider')
        if has_log_provider and logger.logging_provider == logging_provider:
            return logger

        logger.handlers.clear()

        func = self.logger_map.get(logging_provider, self.get_logger_default)

        return func(name, event_handlers)

    def get_logger_default(self, name, event_handlers=[]):

        logger = logging.getLogger(name)
        logger.setLevel(logging.INFO)
        logger.logging_provider = LoggingProvider.DEFAULT

        return logger

    def get_logger_logstash(self, name, event_handlers=[]):
        from logstash import TCPLogstashHandler
        from .providers.logstash import LogstashFormatter

        logging_url = config.get_url()
        logging_port = config.get_port()
        logging_environment = config.get_environment()

        logger = logging.getLogger(name)
        logger.logging_provider = LoggingProvider.LOGSTASH

        logger.setLevel(logging.INFO)
        handler = TCPLogstashHandler(logging_url, logging_port, version=1)
        handler.setFormatter(LogstashFormatter(
            environment=logging_environment,
            event_handlers=event_handlers
        ))
        logger.addHandler(handler)

        return logger

    def get_logger_stackdriver(self, name, event_handlers=[]):
        """
        Docs: https://googleapis.dev/python/logging/latest/handlers.html
        """
        import google.cloud.logging
        from .providers.stackdriver import StackDriverFormatter

        logging_environment = config.get_environment()

        logger = logging.getLogger(name)
        logger.logging_provider = LoggingProvider.STACK_DRIVER

        client = google.cloud.logging.Client()

        handler = client.get_default_handler()

        # Setup logger explicitly with this handler
        handler.setFormatter(StackDriverFormatter(
            environment=logging_environment,
            event_handlers=event_handlers)
        )

        logger.setLevel(logging.INFO)
        logger.addHandler(handler)

        return logger


config = LoggingConfig()
logger_manager = LoggerManager()


def getLogger(name: str, event_handlers=[]):
    """Creates a logger with .

    :type name: str
    :param name: the name of the logger to be constructed.

    :type event_handlers: list
    :param name: list of event handlers

    :rtype: :class:`logging.Logger`
    :returns: Logger created.
    """
    return logger_manager.get_logger(name, event_handlers)


__all__ = [
    'getLogger',
    'config',
]



import unittest

from ..helpers import import_string
from ..helpers import import_string_list
from ..temlogger import config


class TestImportString(unittest.TestCase):

    def test_import_raise_attribute_error(self):

        with self.assertRaises(ImportError) as ctx:
            import_string('temlogger.no_existing_module')

        self.assertIsInstance(ctx.exception, ImportError)

    def test_import_raise_value_error(self):

        with self.assertRaises(ImportError) as ctx:
            import_string('any_module')

        self.assertIsInstance(ctx.exception, ImportError)


class TestImportStringList(unittest.TestCase):

    def test_import_string_list_zero_modules(self):

        modules = import_string_list([])

        self.assertEqual(len(modules), 0)

    def test_import_string_list__one_module(self):

        modules = import_string_list(
            ['temlogger.config']
        )

        self.assertEqual(len(modules), 1)
        self.assertEqual(type(config), type(modules[0]))

    def test_import_string_list_mixed_modules(self):
        def one_more_func():
            pass

        modules = import_string_list([
            'temlogger.config',
            lambda x: x,
            one_more_func,
        ])

        self.assertEqual(len(modules), 3)
        self.assertEqual(type(config), type(modules[0]))
        self.assertEqual(type(one_more_func), type(one_more_func))



import os
import socket
import temlogger
import unittest
import logging
import uuid

from unittest.mock import patch

from .base import clean_temlogger_config
from temlogger.providers.base import FormatterBase


class TestEventHandler(unittest.TestCase):

    def setUp(self):
        """
        Clean config between tests
        """
        clean_temlogger_config()

    @patch.object(FormatterBase, 'format_timestamp',
                  lambda *_: '2020-03-06T21:29:36.246Z')
    def test_register_one_handler_global(self):

        def add_tracker_id(message):
            message['tracker_id'] = 'tracker_id_hex'
            return message

        temlogger.config.set_provider('logstash')
        temlogger.config.setup_event_handlers([add_tracker_id])

        logger = temlogger.getLogger('first_handler')

        format_base = FormatterBase()
        with patch.object(FormatterBase, 'format_with_handlers',
                          wraps=format_base.format_with_handlers) as mock_fwh:

            logger_message = 'First handler message'
            logger.info(logger_message)

            expected = {
                '@timestamp': '2020-03-06T21:29:36.246Z',
                'message': logger_message,
                'host': socket.gethostname(),
                'path': os.path.abspath(__file__),
                'tracker_id': 'tracker_id_hex',
                'environment': '', 'level': 'INFO',
                'logger_name': logger.name,
                'stack_info': None
            }
            mock_fwh.assert_called_once_with(expected)

    def test_register_handler_twice(self):

        def add_tracker_id(message):
            message['tracker_id'] = 'tracker_id_hex'
            return message

        temlogger.config.set_provider('logstash')
        temlogger.config.setup_event_handlers([add_tracker_id])

        self.assertIn(add_tracker_id, temlogger.config.get_event_handlers())
        self.assertEqual(1, len(temlogger.config.get_event_handlers()))

        temlogger.config.setup_event_handlers([lambda x: x])

        self.assertNotIn(add_tracker_id, temlogger.config.get_event_handlers())
        self.assertEqual(1, len(temlogger.config.get_event_handlers()))

    @patch.object(FormatterBase, 'format_timestamp',
                  lambda *_: '2020-03-06T21:29:36.246Z')
    def test_register_two_handler_global(self):
        def tracker_id_param(message):
            message['tracker_id'] = 'tracker_id_hex'
            return message

        def add_request_id_param(message):
            message['request_id'] = 'request_id_hex2'
            return message

        temlogger.config.set_provider('logstash')
        temlogger.config.setup_event_handlers([
            tracker_id_param,
            add_request_id_param
        ])

        logger = temlogger.getLogger('two_handler')

        formatter = logger.handlers[0].formatter
        with patch.object(FormatterBase, 'format_with_handlers',
                          wraps=formatter.format_with_handlers) as mock_fwh:

            logger_message = 'Two handler message'
            logger.info(logger_message)

            expected = {
                '@timestamp': '2020-03-06T21:29:36.246Z',
                'message': logger_message,
                'host': socket.gethostname(),
                'path': os.path.abspath(__file__),
                'tracker_id': 'tracker_id_hex',
                'request_id': 'request_id_hex2',
                'environment': '', 'level': 'INFO',
                'logger_name': logger.name,
                'stack_info': None
            }
            mock_fwh.assert_called_once_with(expected)

    def test_register_event_handler_that_clear_message_keys(self):
        def clear_keys(message):
            message.clear()
            return message

        temlogger.config.set_provider('logstash')
        temlogger.config.setup_event_handlers([clear_keys])

        logger = temlogger.getLogger('three_handler')

        formatter = logger.handlers[0].formatter

        with patch.object(FormatterBase, 'format_with_handlers',
                          wraps=formatter.format_with_handlers) as mock_fwh:
            logger.info('Two handler')

            expected = {}
            mock_fwh.assert_called_once_with(expected)

    @patch.object(FormatterBase, 'format_timestamp',
                  lambda *_: '2020-03-06T21:29:36.246Z')
    def test_register_dotted_string_module(self):
        def clear_keys(message):
            message.clear()
            return message

        temlogger.config.set_provider('logstash')
        temlogger.config.setup_event_handlers([
            'temlogger.tests.base.add_tracker_id_to_message',
        ])

        logger = temlogger.getLogger('four_handler')

        formatter = logger.handlers[0].formatter

        with patch.object(FormatterBase, 'format_with_handlers',
                          wraps=formatter.format_with_handlers) as mock_fwh:

            logger_message = 'Four handler message'
            logger.info(logger_message)

            expected = {
                '@timestamp': '2020-03-06T21:29:36.246Z',
                'message': logger_message,
                'host': socket.gethostname(),
                'path': os.path.abspath(__file__),
                'environment': '', 'level': 'INFO',
                'logger_name': logger.name,
                'stack_info': None,
                'tracker_id_global': 'tracker_id_value_global',
            }
            mock_fwh.assert_called_once_with(expected)

    @patch.object(FormatterBase, 'format_timestamp',
                  lambda *_: '2020-03-06T21:29:36.246Z')
    def test_register_dotted_string_module_and_function(self):
        random_key = uuid.uuid4().hex

        def add_random_key_to_message(message):
            message['random_key'] = random_key
            return message

        temlogger.config.set_provider('logstash')
        temlogger.config.setup_event_handlers([
            'temlogger.tests.base.add_tracker_id_to_message',
            add_random_key_to_message,
        ])

        logger = temlogger.getLogger('four_handler')

        formatter = logger.handlers[0].formatter

        with patch.object(FormatterBase, 'format_with_handlers',
                          wraps=formatter.format_with_handlers) as mock_fwh:

            logger_message = 'Four handler message'
            logger.info(logger_message)

            expected = {
                '@timestamp': '2020-03-06T21:29:36.246Z',
                'message': logger_message,
                'host': socket.gethostname(),
                'path': os.path.abspath(__file__),
                'environment': '', 'level': 'INFO',
                'logger_name': logger.name,
                'stack_info': None,
                'tracker_id_global': 'tracker_id_value_global',
                'random_key': random_key,
            }
            mock_fwh.assert_called_once_with(expected)

    @patch.object(FormatterBase, 'format_timestamp',
                  lambda *_: '2020-03-06T21:29:36.246Z')
    def test_register_event_handler_on_one_logger_only(self):
        random_key = uuid.uuid4().hex

        def add_random_key_to_message_local(message):
            message['random_key'] = random_key
            return message

        temlogger.config.set_provider('logstash')
        logger = temlogger.getLogger('five_handler', event_handlers=[
            add_random_key_to_message_local
        ])

        handler = logger.handlers[0]
        formatter = handler.formatter

        with patch.object(FormatterBase, 'format_with_handlers',
                          wraps=formatter.format_with_handlers) as mock_fwh:

            logger_message = 'Five handler message'
            logger.info(logger_message)

            expected = {
                '@timestamp': '2020-03-06T21:29:36.246Z',
                'message': logger_message,
                'host': socket.gethostname(),
                'path': os.path.abspath(__file__),
                'environment': '',
                'level': 'INFO',
                'logger_name': logger.name,
                'stack_info': None,
                'random_key': random_key,
            }
            mock_fwh.assert_called_once_with(expected)



from setuptools import setup
from setuptools import find_packages
from temlogger import __version__


with open('README.rst', 'r') as fh:
    long_description = fh.read()

setup(
    name='temlogger-test',
    version=__version__,
    description='Python logging handler for Logstash and StackDriver.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    url='https://github.com/tembici/temlogger',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Logging',
    ],
    install_requires=[
        'google-cloud-logging>=1.14.0,<2',
        'python3-logstash==0.4.80'
    ]
)



import json
import unittest
import logging

from .base import clean_temlogger_config
from ..providers.default import DefaultFormatter
from ..providers.logstash import LogstashFormatter
from ..providers.stackdriver import StackDriverFormatter


class TestDefaultFormatter(unittest.TestCase):

    def setUp(self):
        """
        Clean config between tests
        """
        clean_temlogger_config()

    def test_format(self):

        formater = DefaultFormatter(environment='develop')
        log_message = 'Log entry message'

        record = logging.makeLogRecord({'msg': log_message})

        str_message = formater.format(record)
        self.assertIsInstance(str_message, str)

        message = json.loads(str_message)

        self.assertEqual(message['message'], log_message)
        self.assertEqual(message['environment'], 'develop')


class TestLogstashFormatter(unittest.TestCase):

    def setUp(self):
        """
        Clean config between tests
        """
        clean_temlogger_config()

    def test_format(self):

        formater = LogstashFormatter(environment='develop')
        log_message = 'Log entry message'

        record = logging.makeLogRecord({'msg': log_message})

        str_message = formater.format(record)
        message = json.loads(str_message)

        self.assertIsInstance(str_message, str)
        self.assertEqual(message['message'], log_message)
        self.assertEqual(message['environment'], 'develop')


class TestStackDriverFormatter(unittest.TestCase):

    def setUp(self):
        """
        Clean config between tests
        """
        clean_temlogger_config()

    def test_format(self):

        formater = StackDriverFormatter(environment='develop')
        log_message = 'Log entry message'

        record = logging.makeLogRecord({'msg': log_message})

        message = formater.format(record)

        self.assertEqual(message['message'], log_message)
        self.assertEqual(message['environment'], 'develop')
